@@ -42,7 +42,8 @@ typedef enum
   WNCK_WINDOW_STATE_SKIP_TASKLIST          = 1 << 5,
   WNCK_WINDOW_STATE_STICKY                 = 1 << 6,
   WNCK_WINDOW_STATE_HIDDEN                 = 1 << 7,
-  WNCK_WINDOW_STATE_FULLSCREEN             = 1 << 8
+  WNCK_WINDOW_STATE_FULLSCREEN             = 1 << 8,
+  WNCK_WINDOW_STATE_DEMANDS_ATTENTION      = 1 << 9
 } WnckWindowState;
 
 typedef enum
@@ -62,7 +63,8 @@ typedef enum
   WNCK_WINDOW_ACTION_MINIMIZE                = 1 << 12,
   WNCK_WINDOW_ACTION_UNMINIMIZE              = 1 << 13,
   WNCK_WINDOW_ACTION_MAXIMIZE                = 1 << 14,
-  WNCK_WINDOW_ACTION_UNMAXIMIZE              = 1 << 15
+  WNCK_WINDOW_ACTION_UNMAXIMIZE              = 1 << 15,
+  WNCK_WINDOW_ACTION_FULLSCREEN              = 1 << 16
 } WnckWindowActions;
 
 typedef enum
@@ -137,6 +139,8 @@ WnckApplication* wnck_window_get_application  (WnckWindow *window);
 gulong           wnck_window_get_group_leader (WnckWindow *window);
 gulong           wnck_window_get_xid          (WnckWindow *window);
 
+WnckClassGroup *wnck_window_get_class_group (WnckWindow *window);
+
 const char* wnck_window_get_session_id        (WnckWindow *window);
 const char* wnck_window_get_session_id_utf8   (WnckWindow *window);
 int         wnck_window_get_pid               (WnckWindow *window);
@@ -150,14 +154,19 @@ gboolean wnck_window_is_maximized              (WnckWindow *window);
 gboolean wnck_window_is_shaded                 (WnckWindow *window);
 gboolean wnck_window_is_skip_pager             (WnckWindow *window);
 gboolean wnck_window_is_skip_tasklist          (WnckWindow *window);
+gboolean wnck_window_is_fullscreen                 (WnckWindow *window);
 gboolean wnck_window_is_sticky                 (WnckWindow *window);
+gboolean wnck_window_demands_attention         (WnckWindow *window);
 
 void wnck_window_set_skip_pager    (WnckWindow *window,
                                     gboolean skip);
 void wnck_window_set_skip_tasklist (WnckWindow *window,
                                     gboolean skip);
+void wnck_window_set_fullscreen (WnckWindow *window,
+                                 gboolean fullscreen);
 
-void wnck_window_close                   (WnckWindow *window);
+void wnck_window_close                   (WnckWindow *window,
+					  guint32     timestamp);
 void wnck_window_minimize                (WnckWindow *window);
 void wnck_window_unminimize              (WnckWindow *window);
 void wnck_window_maximize                (WnckWindow *window);
@@ -184,6 +193,7 @@ void     wnck_window_unpin     (WnckWindow *window);
 
 void     wnck_window_activate  (WnckWindow *window);
 gboolean wnck_window_is_active (WnckWindow *window);
+gboolean wnck_window_is_most_recently_activated (WnckWindow *window);
 void     wnck_window_activate_transient (WnckWindow *window);
 
 
